@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Base;
 
-our $DATE = '2015-01-04'; # DATE
-our $VERSION = '0.73'; # VERSION
+our $DATE = '2015-01-09'; # DATE
+our $VERSION = '0.74'; # VERSION
 
 use 5.010001;
 use strict;
@@ -43,6 +43,15 @@ has summary => (is=>'rw');
 has tags => (is=>'rw');
 has url => (is=>'rw');
 
+has read_env => (is=>'rw', default=>1);
+has env_name => (
+    is => 'rw',
+    default => sub {
+        my $self = shift;
+        __default_env_name($self->program_name);
+    },
+);
+
 has read_config => (is=>'rw', default=>1);
 has config_filename => (is=>'rw');
 has config_dirs => (
@@ -293,6 +302,16 @@ _
 
 );
 
+sub __default_env_name {
+    my ($prog, $subc) = @_;
+    for ($prog, $subc) {
+        next unless defined;
+        $_ = uc($_);
+        s/[^A-Z0-9]+/_/g;
+    }
+    $prog . (defined($subc) ? "_$subc" : "") . "_OPT";
+}
+
 sub hook_before_read_config_file {}
 
 sub get_meta {
@@ -308,7 +327,7 @@ sub get_meta {
 
 sub get_program_and_subcommand_name {
     my ($self, $r) = @_;
-    my $res = $self->program_name . " " .
+    my $res = ($self->program_name // "") . " " .
         ($r->{subcommand_name} // "");
     $res =~ s/\s+$//;
     $res;
@@ -367,9 +386,43 @@ sub _detect_completion {
         return 1;
     }
 
+    # assume default is bash
+    $r->{shell} //= 'bash';
+
     0;
 }
 
+sub _read_env {
+    my ($self, $r) = @_;
+
+    return [] unless $self->read_env;
+    my $env_name = $self->env_name;
+    my $env = $ENV{$env_name};
+    return [] unless defined $env;
+
+    # XXX is it "proper" to use Complete::* modules to parse cmdline, outside
+    # the context of completion?
+
+    my $words;
+    if ($r->{shell} eq 'bash') {
+        require Complete::Bash;
+        ($words, undef) = @{ Complete::Bash::parse_cmdline($env, 0,'') };
+    } elsif ($r->{shell} eq 'fish') {
+        require Complete::Fish;
+        ($words, undef) = @{ Complete::Fish::parse_cmdline($env) };
+    } elsif ($r->{shell} eq 'tcsh') {
+        require Complete::Tcsh;
+        ($words, undef) = @{ Complete::Tcsh::parse_cmdline($env) };
+    } elsif ($r->{shell} eq 'zsh') {
+        require Complete::Zsh;
+        ($words, undef) = @{ Complete::Zsh::parse_cmdline($env) };
+    } else {
+        die "Unsupported shell '$r->{shell}'";
+    }
+    $log->tracef("words from environment %s (%s): %s", $env_name, $env, $words);
+    $words;
+}
+
 sub do_completion {
     my ($self, $r) = @_;
 
@@ -394,6 +447,12 @@ sub do_completion {
 
     shift @$words; $cword--; # strip program name
 
+    {
+        my $env_words = $self->_read_env($r);
+        unshift @$words, @$env_words;
+        $cword += @$env_words;
+    }
+
     # check whether subcommand is defined. try to search from --cmd, first
     # command-line argument, or default_subcommand.
     {
@@ -461,6 +520,7 @@ sub do_completion {
      {
          "func.words" => $words,
          "func.cword" => $cword,
+         "cmdline.skip_format" => 1,
      }];
 }
 
@@ -482,6 +542,7 @@ sub _read_config {
     my $reader = Config::IOD::Reader->new;
     my %res;
     for my $path (@{ $r->{config_paths} }) {
+        $log->tracef("Reading config file %s ...", $path);
         my $hoh = $reader->read_file($path);
         $r->{read_config_file} = $path;
         for my $section (keys %$hoh) {
@@ -513,8 +574,10 @@ sub _parse_argv1 {
                 $co->{$k}{handler}->($go, $val, $r);
             };
         }
+        #$log->tracef("\@ARGV before parsing common opts: %s", \@ARGV);
         Getopt::Long::GetOptions(@go_spec);
         Getopt::Long::Configure($old_go_conf);
+        #$log->tracef("\@ARGV after  parsing common opts: %s", \@ARGV);
     }
 
     # select subcommand and fill subcommand data
@@ -603,19 +666,21 @@ sub _parse_argv2 {
             my $scn  = $r->{subcommand_name};
             my $profile = $r->{config_profile};
             my $found;
-            for my $section (keys %$conf) {
+            # put GLOBAL before all other sections
+            my @sections = sort {
+                ($a eq 'GLOBAL' ? 0:1) <=> ($b eq 'GLOBAL' ? 0:1) ||
+                    $a cmp $b
+            } keys %$conf;
+            for my $section (@sections) {
                 if (defined $profile) {
                     if (length $scn) {
-                        next unless
-                            $section =~ /\A\Q$scn\E\s+\Q$profile\E\z/ # old, deprecated
-                                || $section =~ /\A\Q$scn\E\s+\Qprofile=$profile\E\z/;
+                        next unless $section =~ /\A(\Q$scn\E|GLOBAL)\s+\Qprofile=$profile\E\z/;
                     } else {
-                        next unless $section eq $profile # old, deprecated
-                            || $section eq "profile=$profile";
+                        next unless $section eq "profile=$profile";
                     }
                 } else {
                     if (length $scn) {
-                        next unless $section eq $scn;
+                        next unless $section eq $scn || $section eq 'GLOBAL';
                     } else {
                         next unless $section eq 'GLOBAL';
                     }
@@ -636,7 +701,6 @@ sub _parse_argv2 {
                 }
                 $log->tracef("args after reading config: %s", \%args);
                 $found++;
-                last;
             }
             if (defined($profile) && !$found &&
                     defined($r->{read_config_file}) &&
@@ -948,6 +1012,11 @@ sub run {
     eval {
         $self->hook_before_run($r);
 
+        {
+            my $env_words = $self->_read_env($r);
+            unshift @ARGV, @$env_words;
+        }
+
         my $parse_res = $self->parse_argv($r);
         if ($parse_res->[0] == 501) {
             # we'll need to send ARGV to the server, because it's impossible to
@@ -1034,10 +1103,139 @@ Perinci::CmdLine::Base - Base class for Perinci::CmdLine{,::Lite}
 
 =head1 VERSION
 
-This document describes version 0.73 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-01-04.
+This document describes version 0.74 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-01-09.
+
+=head1 DESCRIPTION
 
 =for Pod::Coverage ^(.+)$
 
+=head1 PROGRAM FLOW (NORMAL)
+
+If you execute C<run()>, this is what will happen, in order:
+
+=over
+
+=item * Detect if we are running under tab completion mode
+
+This is done by checking the existence of special environment varibles like
+C<COMP_LINE> or C<COMMAND_LINE> (tcsh). If yes, then jump to L</"PROGRAM FLOW
+(TAB COMPLETION)">. Otherwise, continue.
+
+=item * Run hook_before_run, if defined
+
+This hook (and every other hook) will be passed a single argument C<$r>, a hash
+that contains request data (see L</"REQUEST KEYS">).
+
+Some ideas that you can do in this hook: XXX.
+
+=item * Parse command-line arguments (@ARGV) and set C<action>
+
+If C<read_env> attribute is set to true, and there is environment variable
+defined to set default options (see documentation on L<read_env> and C<env_name>
+attributes) then the environment variable is parsed and prepended first to the
+command-line, so it can be parsed together. For example, if your program is
+called C<foo> and environment variable C<FOO_OPT> is set to C<--opt1 --opt2
+val>. When you execute:
+
+ % foo --no-opt1 --trace 1 2
+
+then C<@ARGV> will be set to C<<('--opt1', '--opt2', 'val', '--no-opt1',
+'--trace', 1, 2)>>. This way, command-line arguments can have a higher
+precedence and override setting from the environment variable (in the example,
+C<--opt1> is negated by C<--no-opt1>).
+
+Currently, parsing is done in two steps. The first step is to extract subcommand
+name. Because we want to allow e.g. C<cmd --verbose subcmd> in addition to C<cmd
+subcmd> (that is, user is allowed to specify options before subcommand name) we
+cannot simply get subcommand name from the first element of C<@ARGV> but must
+parse command-line options. Also, we want to allow user specifying subcommand
+name from option C<cmd --cmd subcmd> because we want to support the notion of
+"default subcommand" (subcommand that takes effect if there is no subcommand
+specified).
+
+In the first step, since we do not know the subcommand yet, we only parse common
+options and strip them. Unknown options at this time will be passed through.
+
+If user specifies common option like C<--help> or <--version>, then action will
+be set to (respectively) C<help> and C<version> and the second step will be
+skipped. Otherwise we continue the the second step and action by default is set
+to C<call>.
+
+At the end of the first step, we already know the subcommand name (of course, if
+subcommand name is unknown, we exit with error) along with subcommand spec: its
+URL, per-subcommand settings, and so on (see the C<subcommands> attribute). If
+there are no subcommands, subcommand name is set to C<''> (empty string) and the
+subcommand spec is filled from the attributes, e.g. C<url>, C<summary>, <tags>,
+and so on.
+
+We then perform a C<meta> Riap request to the URL to get the Rinci metadata.
+From the Rinci metadata we get list of arguments (the C<args> property). From
+this, we generate a spec of command-line options to feed to L<Getopt::Long>.
+There are some conversions being done, e.g. an argument called C<foo_bar> will
+become command-line option C<--foo-bar>. Command-line aliases from metadata are
+also added to the C<Getopt::Long> spec.
+
+It is also at this step that we read config file (if C<read_config> attribute is
+true). We run B<hook_before_read_config_file> first. Some ideas to do in this
+hook: setting default config profile.
+
+We then pass the spec to C<Getopt::Long::GetOptions>, we get function arguments.
+
+We then run B<hook_after_parse_argv>. Some ideas to do in this hook: XXX.
+
+Function arguments that are still missing can be filled from STDIN or files, if
+the metadata specifies C<cmdline_src> property (see L<Rinci::function> for more
+details).
+
+=item * Delegate to C<run_$action> method
+
+After we get the action from the previous step, we delegate to separate
+C<run_$action> method (so there is C<run_version>, C<run_help>, and so on; and
+also C<run_call>). These methods also receive C<$r> as their argument and must
+return an enveloped result (see L<Rinci::function> for more details).
+
+Result is put in C<< $r->{res} >>.
+
+=item * Run hook_format_result
+
+Hook must set C<< $r->{fres} >> (formatted result).
+
+If result has C<cmdline.skip_format> result metadata property, then this step is
+skipped and C<< $r->{fres} >> is simply taken from C<< $r->{res}[2] >>.
+
+A C<run_$action> method can set
+
+=item * Run hook_display_result
+
+This hook is used by XXX.
+
+=item * Run hook_after_run, if defined
+
+Some ideas to do in this hook: XXX.
+
+=item * Exit (or return result)
+
+If C<exit> attribute is true, will C<exit()> with the action's envelope result
+status. If status is 200, exit code is 0. Otherwise exit code is status minus
+300. So, a response C<< [501, "Not implemented"] >> will result in exit code of
+201.
+
+If C<exit> attribute is false, will simply return the action result (C<<
+$r->{res} >>). And will also set exit code in C<<
+$r->{res}[3]{'x.perinci.cmdline.base.exit_code'} >>.
+
+=back
+
+=head1 PROGRAM FLOW (TAB COMPLETION)
+
+If program is detected running in tab completion mode, there is some differences
+in the flow. First, C<@ARGV> is set from C<COMP_LINE> (or C<COMMAND_LINE>)
+environment variable. Afterwards, completion is done by calling
+L<Perinci::Sub::Complete>'s C<complete_cli_arg>.
+
+The result is then output to STDOUT (resume from Run hook_format_result step in
+the normal program flow).
+
 =head1 REQUEST KEYS
 
 The various values in the C<$r> hash/stash.
@@ -1143,6 +1341,76 @@ If we are doing streaming argument, this will be set to the argument name.
 
 =back
 
+=head1 CONFIGURATION
+
+Configuration can be used to set function arguments.
+
+Configuration is currently in the L<IOD> (basically INI) format.
+
+By default these paths are searched: C<$HOME/$prog_name.conf>,
+C</etc/$prog_name.conf>. The location can be customized from command-line option
+C<--config-path>.
+
+All existing configuration files will be read in order.
+
+For application that does not have subcommand, you can put parameters outside
+any section, e.g.:
+
+ param=val
+ otherparam=val
+ ...
+
+For application that has subcommands, put parameters inside section with the
+same name as the subcommand name:
+
+ [subcommand1]
+ param=val
+ ...
+
+ [subcommand2]
+ param2=val
+ ...
+
+Or you can also put some parameters outside the section which will be used for
+all subcommands:
+
+ commonarg=val
+
+ [subcommand1]
+ param1=val
+ ...
+
+ [subcommand2]
+ param2=val
+ ...
+
+A configuration file can also have (multiple) profiles, to allow multiple
+configuration to be stored in a single file. Example:
+
+ [profile=p1]
+ a=1
+ b=2
+
+ [profile=p2]
+ a=10
+ b=20
+
+ [subcommand1 profile=p1]
+ c=3
+
+ [subcommand1 profile=p2]
+ c=1
+
+If you run:
+
+ % cmd subcommand1 --config-profile p1
+
+then your subcommand1 function will get: a=1, b=2, c=3. If you run:
+
+ % cmd subcommand1 --config-profile p2
+
+then your subcommand1 function will get: a=10, b=20, c=30.
+
 =head1 ATTRIBUTES
 
 =head2 actions => array
@@ -1427,6 +1695,17 @@ entity.
 Alternatively you can provide multiple functions from which the user can select
 using the first argument (see B<subcommands>).
 
+=head2 read_env => bool (default: 1)
+
+Whether to read environment variable for default options.
+
+=head2 env_name => str
+
+Environment name to read default options from. Default is from program name,
+upper-cased, sequences of dashes/nonalphanums replaced with a single underscore,
+plus a C<_OPT> suffix. So if your program name is called C<cpandb-cpanmeta> the
+default environment name is C<CPANDB_CPANMETA_OPT>.
+
 =head2 read_config => bool (default: 1)
 
 Whether to read configuration files.
@@ -1447,21 +1726,8 @@ C<foo-bar.conf>.
 
 =head2 $cmd->run() => ENVRES
 
-Will parse command-line arguments with C<parse_argv()>, select/set subcommand,
-call hooks, run the appropriate C<run_ACTION()> method, and finally format and
-display the result.
-
-The C<run_ACTION()> methods will be passed C<$r> and is supposed to return an
-enveloped result. The result will then be put in C<< $r->{res} >>.
-
-If C<exit> attribute is true, will exit with the action's envelope result
-status. If status is 200, exit code is 0. Otherwise exit code is status minus
-300. So, a response C<< [501, "Not implemented"] >> will result in exit code of
-201.
-
-If C<exit> attribute is false, will simply return the action result (C<<
-$r->{res} >>). And will also return the exit code in C<<
-$r->{res}[3]{'x.perinci.cmdline.base.exit_code'} >>.
+The main method to run your application. See L</"PROGRAM FLOW"> for more details
+on what this method does.
 
 =head2 $cmd->do_completion() => ENVRES
 
@@ -1619,7 +1885,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Cm
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/sharyanto/perl-Perinci-CmdLine-Lite>.
+Source repository is at L<https://github.com/perlancar/perl-Perinci-CmdLine-Lite>.
 
 =head1 BUGS
 
